@@ -17,7 +17,7 @@
  * See the COPYING file in the top-level directory.
  */
 
-#include "glib-compat.h"
+#include <glib.h>
 
 #include <string.h>
 #include <stdbool.h>
@@ -26,6 +26,7 @@
 #include "cac-aca.h"
 #include "vcard.h"
 #include "vcard_emul.h"
+#include "vcardt_internal.h"
 #include "card_7816.h"
 #include "simpletlv.h"
 #include "common.h"
@@ -260,6 +261,7 @@ cac_common_process_apdu(VCard *card, VCardAPDU *apdu, VCardResponse **response)
     unsigned int i;
 
     applet_private = vcard_get_current_applet_private(card, apdu->a_channel);
+    g_assert(applet_private);
 
     switch (apdu->a_ins) {
     case CAC_GET_PROPERTIES:
@@ -280,6 +282,7 @@ cac_common_process_apdu(VCard *card, VCardAPDU *apdu, VCardResponse **response)
             /* If P1 = 0x00 cannot be supported by the smart card, SW1 = 0x6A and SW2 = 86. */
             *response = vcard_make_response(
                         VCARD7816_STATUS_ERROR_P1_P2_INCORRECT);
+            ret = VCARD_DONE;
             break;
         case 0x01:
             /* Get all the properties. */
@@ -293,6 +296,7 @@ cac_common_process_apdu(VCard *card, VCardAPDU *apdu, VCardResponse **response)
 
             *response = get_properties(card, applet_private->properties,
                 applet_private->properties_len, NULL, 0, apdu->a_Le);
+            ret = VCARD_DONE;
 
             break;
         case 0x02:
@@ -306,6 +310,7 @@ cac_common_process_apdu(VCard *card, VCardAPDU *apdu, VCardResponse **response)
             }
             *response = get_properties(card, applet_private->properties,
                 applet_private->properties_len, apdu->a_body, apdu->a_Lc, apdu->a_Le);
+            ret = VCARD_DONE;
             break;
         case 0x40:
             /* XXX This is undocumented P1 argument, which returns properties
@@ -313,15 +318,16 @@ cac_common_process_apdu(VCard *card, VCardAPDU *apdu, VCardResponse **response)
              */
             *response = get_properties(card, applet_private->properties,
                 applet_private->long_properties_len, NULL, 0, apdu->a_Le);
+            ret = VCARD_DONE;
             break;
         default:
             /* unknown params returns (SW1=0x6A, SW2=0x86) */
             *response = vcard_make_response(
                         VCARD7816_STATUS_ERROR_P1_P2_INCORRECT);
+            ret = VCARD_DONE;
             break;
         }
 
-        ret = VCARD_DONE;
         break;
     case VCARD7816_INS_SELECT_FILE:
         if (apdu->a_p1 != 0x02) {
@@ -382,6 +388,7 @@ cac_common_process_apdu_read(VCard *card, VCardAPDU *apdu,
     int size, offset;
 
     applet_private = vcard_get_current_applet_private(card, apdu->a_channel);
+    g_assert(applet_private);
 
     switch (apdu->a_ins) {
     case CAC_READ_BUFFER:
@@ -492,7 +499,7 @@ cac_applet_pki_process_apdu(VCard *card, VCardAPDU *apdu,
     CACPKIAppletData *pki_applet;
     VCardAppletPrivate *applet_private;
     int size;
-    unsigned char *sign_buffer;
+    unsigned char *sign_buffer = NULL;
     bool retain_sign_buffer = FALSE;
     vcard_7816_status_t status;
     VCardStatus ret = VCARD_FAIL;
@@ -518,7 +525,7 @@ cac_applet_pki_process_apdu(VCard *card, VCardAPDU *apdu,
 
         sign_buffer = g_realloc(pki_applet->sign_buffer,
                                 pki_applet->sign_buffer_len + size);
-        memcpy(sign_buffer+pki_applet->sign_buffer_len, apdu->a_body, size);
+        memcpy(sign_buffer + pki_applet->sign_buffer_len, apdu->a_body, size);
         size += pki_applet->sign_buffer_len;
         switch (apdu->a_p1) {
         case  0x80:
@@ -539,7 +546,7 @@ cac_applet_pki_process_apdu(VCard *card, VCardAPDU *apdu,
                 break;
             }
             *response = vcard_response_new(card, sign_buffer, size, apdu->a_Le,
-                                                     VCARD7816_STATUS_SUCCESS);
+                                           VCARD7816_STATUS_SUCCESS);
             if (*response == NULL) {
                 *response = vcard_make_response(
                                 VCARD7816_STATUS_EXC_ERROR_MEMORY_FAILURE);
@@ -688,15 +695,6 @@ cac_applet_aca_process_apdu(VCard *card, VCardAPDU *apdu,
         }
         ret = VCARD_DONE;
         break;
-    /* XXX unknown APDU from ActivClient after PIN verification */
-    case 0x5A:
-        /* ???
-         * 80 5A 00 00 00
-         * HW response: 90 00
-         */
-        *response = vcard_make_response(VCARD7816_STATUS_SUCCESS);
-        ret = VCARD_DONE;
-        break;
     default:
         ret = cac_common_process_apdu(card, apdu, response);
         break;
@@ -837,8 +835,8 @@ static VCardAppletPrivate *
 cac_new_pki_applet_private(int i, const unsigned char *cert,
                            int cert_len, VCardKey *key)
 {
-    CACPKIAppletData *pki_applet_data;
-    VCardAppletPrivate *applet_private;
+    CACPKIAppletData *pki_applet_data = NULL;
+    VCardAppletPrivate *applet_private = NULL;
     int bits;
 
     /* PKI applet Properies ex.:
@@ -1024,8 +1022,7 @@ cac_new_pki_applet_private(int i, const unsigned char *cert,
     return applet_private;
 
 failure:
-    if (applet_private)
-        cac_delete_pki_applet_private(applet_private);
+    cac_delete_pki_applet_private(applet_private);
     return NULL;
 }
 
@@ -1453,7 +1450,7 @@ failure:
 static VCardAppletPrivate *
 cac_new_ccc_applet_private(int cert_count)
 {
-    VCardAppletPrivate *applet_private;
+    VCardAppletPrivate *applet_private = NULL;
 
     /* CCC applet Properties ex.:
      * 01  Tag: Applet Information
@@ -1754,9 +1751,7 @@ cac_new_ccc_applet_private(int cert_count)
     return applet_private;
 
 failure:
-    if (applet_private) {
-        cac_delete_ccc_applet_private(applet_private);
-    }
+    cac_delete_ccc_applet_private(applet_private);
     return NULL;
 }
 
@@ -1795,8 +1790,8 @@ failure:
 static VCardAppletPrivate *
 cac_new_aca_applet_private(int cert_count)
 {
-    CACACAAppletData *aca_applet_data;
-    VCardAppletPrivate *applet_private;
+    CACACAAppletData *aca_applet_data = NULL;
+    VCardAppletPrivate *applet_private = NULL;
 
     /* ACA applet Properties ex.:
      * 01  Tag: Applet Information
@@ -1889,8 +1884,6 @@ cac_new_aca_applet_private(int cert_count)
     /* Create the private data structure */
     applet_private = g_new0(VCardAppletPrivate, 1);
     aca_applet_data = &(applet_private->u.aca_data);
-    if (applet_private == NULL)
-        goto failure;
 
     /* store the applet OID */
     applet_private->coids = g_malloc(sizeof(struct coid));
@@ -1906,12 +1899,6 @@ cac_new_aca_applet_private(int cert_count)
     aca_applet_data->pki_applets = cert_count;
 
     return applet_private;
-
-failure:
-    if (applet_private != NULL) {
-       cac_delete_aca_applet_private(applet_private);
-    }
-    return NULL;
 }
 
 static VCardAppletPrivate *
@@ -1982,18 +1969,22 @@ cac_new_empty_applet_private(unsigned char objects[][2], unsigned int objects_le
 
     /* Create the private data structure */
     applet_private = g_new0(VCardAppletPrivate, 1);
-    if (applet_private == NULL)
-        goto failure;
 
     /* Create Object ID list */
-    applet_private->coids = g_malloc_n(objects_len, sizeof(struct coid));
-    memcpy(applet_private->coids, objects, 2*objects_len);
-    applet_private->coids_len = objects_len;
+    if (objects_len > 0) {
+        applet_private->coids = g_memdup(objects, sizeof(struct coid) * objects_len);
+        applet_private->coids_len = objects_len;
+    }
 
     /* Clone the properties */
     applet_private->properties_len = properties_len;
     applet_private->long_properties_len = properties_len; /*TODO*/
     applet_private->properties = simpletlv_clone(properties, properties_len);
+
+    /* Avoid dangling pointers in the static structure */
+    properties[0].value.value = NULL;
+    properties[1].value.value = NULL;
+
     if (applet_private->properties == NULL)
         goto failure;
 
@@ -2021,7 +2012,7 @@ failure:
 }
 
 static VCardAppletPrivate *
-cac_new_passthrough_applet_private(VCard *card, const char *label,
+cac_new_passthrough_applet_private(G_GNUC_UNUSED VCard *card, const char *label,
                                    const unsigned char *aid, unsigned int aid_len)
 {
     CACPTAppletData *pt_applet_data;
@@ -2143,6 +2134,8 @@ cac_new_pki_applet(int i, const unsigned char *cert,
     unsigned char pki_aid[] = { 0xa0, 0x00, 0x00, 0x00, 0x79, 0x01, 0x00 };
     int pki_aid_len = sizeof(pki_aid);
 
+    g_debug("%s: called", __func__);
+
     pki_aid[pki_aid_len-1] = i;
 
     applet_private = cac_new_pki_applet_private(i, cert, cert_len, key);
@@ -2230,13 +2223,33 @@ failure:
     return NULL;
 }
 
+/*
+ * Get ATR of CAC card since some application might be picky about out
+ * virtual one
+ */
+static unsigned char cac_atr[] = {
+    /* The older CAC card from official CAC document */
+    0x3B, 0x7A, 0x18, 0x00, 0x00, 0x73, 0x66, 0x74, 0x65, 0x20, 0x63, 0x64, 0x31, 0x34, 0x34
+};
+static int cac_atr_len = sizeof(cac_atr);
+
+static void
+cac_get_atr(G_GNUC_UNUSED VCard *card, unsigned char *atr, int *atr_len)
+{
+    int len;
+    assert(atr != NULL);
+
+    len = MIN(cac_atr_len, *atr_len);
+    memcpy(atr, cac_atr, len);
+    *atr_len = len;
+}
+
 /*
  * Initialize the cac card. This is the only public function in this file. All
  * the rest are connected through function pointers.
  */
 VCardStatus
-cac_card_init(VReader *reader, VCard *card,
-              const char *params,
+cac_card_init(G_GNUC_UNUSED VReader *reader, VCard *card,
               unsigned char * const *cert,
               int cert_len[],
               VCardKey *key[] /* adopt the keys*/,
@@ -2252,6 +2265,8 @@ cac_card_init(VReader *reader, VCard *card,
         {0x90, 0x00},
     };
 
+    g_debug("%s: called", __func__);
+
     /* CAC Cards are VM Cards */
     vcard_set_type(card, VCARD_VM);
 
@@ -2386,6 +2401,9 @@ cac_card_init(VReader *reader, VCard *card,
 
     /* GP applet is created from vcard_emul_type() */
 
+    /* Modify ATR to match existing cards */
+    vcard_set_atr_func(card, cac_get_atr);
+
     return VCARD_DONE;
 
 failure:
