@@ -11,6 +11,8 @@
  */
 #include "config.h"
 
+#include <glib.h>
+
 /*
  * NSS headers
  */
@@ -29,8 +31,6 @@
 #include <secmodt.h>
 #include <sechash.h>
 
-#include "glib-compat.h"
-
 #include "vcard.h"
 #include "card_7816t.h"
 #include "vcard_emul.h"
@@ -52,7 +52,6 @@ typedef enum {
 struct VCardKeyStruct {
     CERTCertificate *cert;
     PK11SlotInfo *slot;
-    SECKEYPrivateKey *key;
     VCardEmulTriState failedX509;
 };
 
@@ -87,11 +86,11 @@ enum {
 };
 
 struct VCardEmulOptionsStruct {
-    void *nss_db;
+    char *nss_db;
     VirtualReaderOptions *vreader;
     int vreader_count;
     VCardEmulType hw_card_type;
-    const char *hw_type_params;
+    char *hw_type_params;
     int use_hw;
 };
 
@@ -155,10 +154,6 @@ vcard_emul_make_key(PK11SlotInfo *slot, CERTCertificate *cert)
     key = g_new(VCardKey, 1);
     key->slot = PK11_ReferenceSlot(slot);
     key->cert = CERT_DupCertificate(cert);
-    /* NOTE: if we aren't logged into the token, this could return NULL */
-    /* NOTE: the cert is a temp cert, not necessarily the cert in the token,
-     * use the DER version of this function */
-    key->key = PK11_FindKeyByDERCert(slot, cert, NULL);
     key->failedX509 = VCardEmulUnknown;
     return key;
 }
@@ -170,10 +165,6 @@ vcard_emul_delete_key(VCardKey *key)
     if (!nss_emul_init || (key == NULL)) {
         return;
     }
-    if (key->key) {
-        SECKEY_DestroyPrivateKey(key->key);
-        key->key = NULL;
-    }
     if (key->cert) {
         CERT_DestroyCertificate(key->cert);
     }
@@ -189,12 +180,8 @@ vcard_emul_delete_key(VCardKey *key)
 static SECKEYPrivateKey *
 vcard_emul_get_nss_key(VCardKey *key)
 {
-    if (key->key) {
-        return key->key;
-    }
     /* NOTE: if we aren't logged into the token, this could return NULL */
-    key->key = PK11_FindPrivateKeyFromCert(key->slot, key->cert, NULL);
-    return key->key;
+    return PK11_FindPrivateKeyFromCert(key->slot, key->cert, NULL);
 }
 
 /*
@@ -276,7 +263,8 @@ vcard_emul_rsa_op(VCard *card, VCardKey *key,
      */
     signature_len = PK11_SignatureLen(priv_key);
     if ((unsigned)buffer_size != signature_len) {
-        return  VCARD7816_STATUS_ERROR_DATA_INVALID;
+        ret = VCARD7816_STATUS_ERROR_DATA_INVALID;
+        goto cleanup;
     }
     /* be able to handle larger keys if necessary */
     bp = &buf[0];
@@ -373,7 +361,7 @@ vcard_emul_rsa_op(VCard *card, VCardKey *key,
     pad_len -= 3; /* format is 0 || 2 || pad || 0 || data */
     /*
      * padding for PKCS #1 encrypted data is a string of random bytes. The
-     * random butes protect against potential decryption attacks against RSA.
+     * random bytes protect against potential decryption attacks against RSA.
      * Since PrivDecrypt has already stripped those bytes, we can't reconstruct
      * them. This shouldn't matter to the upper level code which should just
      * strip this code out anyway, so We'll pad with a constant 3.
@@ -393,6 +381,7 @@ cleanup:
     if (bp != buf) {
         g_free(bp);
     }
+    SECKEY_DestroyPrivateKey(priv_key);
     return ret;
 }
 
@@ -880,7 +869,7 @@ static const VCardEmulOptions default_options = {
     .vreader = NULL,
     .vreader_count = 0,
     .hw_card_type = VCARD_EMUL_CAC,
-    .hw_type_params = "",
+    .hw_type_params = NULL,
     .use_hw = USE_HW_YES,
 };
 
@@ -955,11 +944,11 @@ vcard_emul_force_card_insert(VReader *vreader)
  * vcards can be created (indicates error with certificates provided
  * or db), or if any other higher level error (NSS error, missing coolkey). */
 static int vcard_emul_init_called;
+static NSSInitContext *nss_ctx = NULL;
 
 VCardEmulError
 vcard_emul_init(const VCardEmulOptions *options)
 {
-    SECStatus rv;
     PRBool has_readers = PR_FALSE;
     VReader *vreader;
     VReaderEmul *vreader_emul;
@@ -967,6 +956,8 @@ vcard_emul_init(const VCardEmulOptions *options)
     SECMODModuleList *module_list;
     SECMODModuleList *mlp;
     int i;
+    gchar *path = NULL;
+    const gchar *nss_db;
 
     g_debug("%s: called", __func__);
 
@@ -1000,28 +991,31 @@ vcard_emul_init(const VCardEmulOptions *options)
 #endif
 
     /* first initialize NSS */
-    if (options->nss_db) {
-        rv = NSS_Init(options->nss_db);
-    } else {
-        gchar *path;
+    nss_db = options->nss_db;
+    if (nss_db == NULL) {
 #ifndef _WIN32
-        path = g_strdup("/etc/pki/nssdb");
+        nss_db = "/etc/pki/nssdb";
 #else
-        if (g_get_system_config_dirs() == NULL ||
-            g_get_system_config_dirs()[0] == NULL) {
+        const gchar * const *config_dirs = g_get_system_config_dirs();
+        if (config_dirs == NULL || config_dirs[0] == NULL) {
             return VCARD_EMUL_FAIL;
         }
 
-        path = g_build_filename(
-            g_get_system_config_dirs()[0], "pki", "nssdb", NULL);
+        path = g_build_filename(config_dirs[0], "pki", "nssdb", NULL);
+        nss_db = path;
 #endif
-
-        rv = NSS_Init(path);
-        g_free(path);
     }
-    if (rv != SECSuccess) {
+
+    nss_ctx = NSS_InitContext(nss_db, "", "", "", NULL, NSS_INIT_READONLY);
+    if (nss_ctx == NULL) {
+        g_debug("%s: NSS_InitContext failed. Does the DB directory '%s' exist?",
+                __func__, nss_db);
+        g_free(path);
         return VCARD_EMUL_FAIL;
     }
+    g_free(path);
+    path = NULL;
+
     /* Set password callback function */
     PK11_SetPasswordFunc(vcard_emul_get_password);
 
@@ -1076,6 +1070,7 @@ vcard_emul_init(const VCardEmulOptions *options)
             vreader_free(vreader);
             has_readers = PR_TRUE;
         }
+        PK11_FreeSlot(slot);
         g_free(certs);
         g_free(cert_len);
         g_free(keys);
@@ -1143,6 +1138,7 @@ vcard_emul_init(const VCardEmulOptions *options)
                         vreader_get_name(vreader));
                 vcard_free(vcard);
             }
+            vreader_free(vreader);
         }
         vcard_emul_new_event_thread(module);
     }
@@ -1172,6 +1168,21 @@ vcard_emul_replay_insertion_events(void)
     vreader_list_delete(list);
 }
 
+VCardEmulError
+vcard_emul_finalize(void)
+{
+    SECStatus rv;
+
+    rv = NSS_ShutdownContext(nss_ctx);
+    if (rv != SECSuccess) {
+        g_debug("%s: NSS_ShutdownContext failed.", __func__);
+        return VCARD_EMUL_FAIL;
+    }
+    nss_ctx = NULL;
+
+    return VCARD_EMUL_OK;
+}
+
 /*
  *  Silly little functions to help parsing our argument string
  */
@@ -1223,9 +1234,9 @@ static VCardEmulOptions options;
 #define NEXT_TOKEN(token) \
             (token) = args; \
             args = strpbrk(args, ",)"); \
-            if (*args == 0 || *args == ')') { \
+            if (args == NULL || *args == 0 || *args == ')') { \
                 fprintf(stderr, "Error: invalid soft specification.\n"); \
-                return NULL; \
+                goto fail; \
             } \
             (token##_length) = args - (token); \
             args = strip(args+1);
@@ -1233,7 +1244,7 @@ static VCardEmulOptions options;
 VCardEmulOptions *
 vcard_emul_options(const char *args)
 {
-    int reader_count = 0;
+    int i, j, reader_count = 0;
     VCardEmulOptions *opts;
 
     /* Allow the future use of allocating the options structure on the fly */
@@ -1257,13 +1268,13 @@ vcard_emul_options(const char *args)
             size_t type_params_length;
             char type_str[100];
             VCardEmulType type;
-            int count, i;
+            int count;
             VirtualReaderOptions *vreaderOpt;
 
             args = strip(args + 5);
             if (*args != '(') {
                 fprintf(stderr, "Error: invalid soft specification.\n");
-                return NULL;
+                goto fail;
             }
             args = strip(args+1);
 
@@ -1277,14 +1288,14 @@ vcard_emul_options(const char *args)
             if (type == VCARD_EMUL_NONE) {
                 fprintf(stderr, "Error: invalid smartcard type '%s'.\n",
                         type_str);
-                return NULL;
+                goto fail;
             }
 
             NEXT_TOKEN(type_params)
 
             if (*args == 0) {
                 fprintf(stderr, "Error: missing cert specification.\n");
-                return NULL;
+                goto fail;
             }
 
             if (opts->vreader_count >= reader_count) {
@@ -1329,12 +1340,17 @@ vcard_emul_options(const char *args)
             if (opts->hw_card_type == VCARD_EMUL_NONE) {
                 fprintf(stderr, "Error: invalid smartcard type '%s'.\n",
                         args);
-                return NULL;
+                goto fail;
             }
             args = find_blank(args);
         /* hw_params= */
         } else if (strncmp(args, "hw_params=", 10) == 0) {
             const char *params;
+
+            if (opts->hw_type_params != NULL) {
+                fprintf(stderr, "Error: redefinition of hw_params= is not allowed.\n");
+                goto fail;
+            }
             args = strip(args+10);
             params = args;
             args = find_blank(args);
@@ -1342,17 +1358,22 @@ vcard_emul_options(const char *args)
         /* db="/data/base/path" */
         } else if (strncmp(args, "db=", 3) == 0) {
             const char *db;
+
+            if (opts->nss_db != NULL) {
+                fprintf(stderr, "Error: redefinition of db= is not allowed.\n");
+                goto fail;
+            }
             args = strip(args+3);
             if (*args != '"') {
                 fprintf(stderr, "Error: you must quote the file path.\n");
-                return NULL;
+                goto fail;
             }
             args++;
             db = args;
             args = strpbrk(args, "\"\n");
             if (args == NULL) {
-              fprintf(stderr, "Error: invalid db argument.\n");
-              return NULL;
+                fprintf(stderr, "Error: invalid db argument.\n");
+                goto fail;
             }
             opts->nss_db = g_strndup(db, args-db);
             if (*args != 0) {
@@ -1370,11 +1391,27 @@ vcard_emul_options(const char *args)
 #endif
         } else {
             fprintf(stderr, "Error: Unknown smartcard specification.\n");
-            return NULL;
+            goto fail;
         }
     } while (*args != 0);
 
     return opts;
+
+fail:
+    /* Clean up what was allocated above on failure */
+    for (i = 0; i < opts->vreader_count; i++) {
+        g_free(opts->vreader[i].name);
+        g_free(opts->vreader[i].vname);
+        g_free(opts->vreader[i].type_params);
+        for (j = 0; j < opts->vreader[i].cert_count; j++) {
+            g_free(opts->vreader[i].cert_name[j]);
+        }
+        g_free(opts->vreader[i].cert_name);
+    }
+    g_free(opts->vreader);
+    g_free(opts->hw_type_params);
+    g_free(opts->nss_db);
+    return NULL;
 }
 
 unsigned char *
@@ -1438,7 +1475,7 @@ vcard_emul_usage(void)
 " db={nss_database}               (default sql:/etc/pki/nssdb)\n"
 " use_hw=[yes|no|removable]       (default yes)\n"
 " hw_type={card_type_to_emulate}  (default CAC)\n"
-" hw_param={param_for_card}       (default \"\")\n"
+" hw_params={param_for_card}      (default \"\")\n"
 " nssemul                         (alias for use_hw=yes, hw_type=CAC)\n"
 #if defined(ENABLE_PCSC)
 " passthru                        (alias for use_hw=yes, hw_type=PASSTHRU)\n"
@@ -1457,7 +1494,7 @@ vcard_emul_usage(void)
 "\n"
 "Unless use_hw is set to no, all tokens that look like removable hardware\n"
 "tokens will be presented to the guest using the emulator specified by\n"
-"hw_type, and parameters of hw_param. If use_hw is set to 'removable', "
+"hw_type, and parameters of hw_params. If use_hw is set to 'removable', "
 "present any removable token.\n"
 "\n"
 "If more one or more soft= parameters are specified, these readers will be\n"
