@@ -10,6 +10,8 @@
  * See the COPYING file in the top-level directory.
  */
 
+#include <glib.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,8 +36,6 @@
 # endif
 #endif
 
-#include "glib-compat.h"
-
 #include "vscard_common.h"
 
 #include "vreader.h"
@@ -150,7 +150,7 @@ static void pcsc_deinit(void)
 
 static GIOChannel *channel_socket;
 static GByteArray *socket_to_send;
-static CompatGMutex socket_to_send_lock;
+static GMutex socket_to_send_lock;
 static guint socket_tag;
 
 static void
@@ -217,8 +217,8 @@ send_msg(
 }
 
 static VReader *pending_reader;
-static CompatGMutex pending_reader_lock;
-static CompatGCond pending_reader_condition;
+static GMutex pending_reader_lock;
+static GCond pending_reader_condition;
 
 #define MAX_ATR_LEN 40
 static gpointer
@@ -797,11 +797,6 @@ main(
         return 1;
     }
 #endif
-#if !GLIB_CHECK_VERSION(2, 31, 0)
-    if (!g_thread_supported()) {
-         g_thread_init(NULL);
-    }
-#endif
 
     while ((c = getopt(argc, argv, "c:e:d:p")) != -1) {
         if (c == '?') {
@@ -866,6 +861,10 @@ main(
     if (emul_args) {
         command_line_options = vcard_emul_options(emul_args);
     }
+    if (cert_count > 0) {
+        g_free(emul_args);
+        emul_args = NULL;
+    }
 
     qemu_host = g_strdup(argv[argc - 2]);
     qemu_port = g_strdup(argv[argc - 1]);
@@ -928,5 +927,7 @@ main(
     pcsc_deinit();
 #endif
 
+    vcard_emul_finalize();
+
     return 0;
 }
