@@ -46,6 +46,9 @@ void test_get_response(void);
 
 void check_login_count(void);
 
+void test_msft_applet(void);
+void test_gp_applet(void);
+
 int isHWTests(void);
 void setHWTests(int);
 
